# MenuBar -- menu bar widget
# Bert Bos <bert@let.rug.nl>
# $Ident$

@class XfwfMenuBar (XfwfRowCol) @file=MenuBar

@ The MenuBar widget is a special kind of RowCol widget. It assumes
that all its children are PullDown buttons or other widgets that pop
up a menu. (But nothing will break if they are not.)  By default, the
children are arranged in a row, wrapping to the next row if the width
is not enough for all of them. But other arrangements are possible,
see the resources of the XfwfRowCol(3) widget.

The MenuBar attaches itself to the |popup| callback of the menu shell
that is popped up by the children, and inserts itself in the list of
pointer grabs that Xt maintains. This is necessary to be able to drag
the mouse from one menu into another, popping down the first one and
popping up the second. If you use a RowCol widget instead of a
MenuBar, you will find that switching from one menu to the next
involves releasing the mouse and then pressing it again.

To find the menu shell, the children are asked for their |popup|
resource. PullDown buttons have this resource, others may not.

The MenuBar widget introduces no new resources.

@PUBLIC

@var frameType = XfwfRaised

@PRIVATE

@ When the |menu_callback| function is called, it tries to get the ID
of the popped up widget.  If it succeeds, the ID is stored in a
private variable, so that the widget can be popped down again, if
another menu pops up.

@var Widget current_menu

@METHODS

@ The |current_menu| variable is intialized to |NULL| by the
|initialize| method.

@proc initialize
{
    $current_menu = NULL;
}

@ When a new child is added, it is added normally by the superclass's
method, but then a callback is added for the |activate| of
the child and the |popdownCallback| of the child's menu. This
callback is defined in the Shell widget. The child's menu is named
|popup| in Pulldown buttons, thus it will only work for them or their
subclasses.

@proc insert_child
{
    Widget popup = NULL;

    #insert_child(child);
    XtVaGetValues(child, XtNpopup, &popup, NULL);
    if (popup) {
	XtAddCallback(child, XtNactivate, menu_popup_cb, $);
	XtAddCallback(popup, XtNpopdownCallback, menu_popdown_cb, $);
    }
}

@TRANSLATIONS

@ Since the MenuBar widget adds itself to the list of cascaded pop
ups that is maintained by Xt, it will receive pointer events when a
menu is popped up. If the event is a button release, the menu will
have to be popped down.

@trans <Btn1Up>: menu_popdown()

@ACTIONS

@ The |menu_popdown| action calls |XtPopdown| for the current menu,
if there is one.

@proc menu_popdown
{
    if ($current_menu != NULL) {
	XtPopdown($current_menu);
	$current_menu = NULL;
    }
}

@UTILITIES

@var char rcsid[] = "$Ident$"

@ The callback |menu_popup_cb| is attached to the |activate| callback
of a (PullDown) button. The |menu_popup_cb| routine establishes a new
grab, overriding the one exerted by the menu, and adds the MenuBar
widget in the list of cascaded menus that is maintained by Xt. This
will ensure that pointer events continue to arrive at the MenuBar
widget.

@proc menu_popup_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
    Widget $ = (Widget) client_data, menu = NULL;
    Cursor cursor;

    XtVaGetValues(w, XtNpopup, &menu, NULL);
    if (menu && $current_menu == menu) return;	/* Already popped up */
    if ($current_menu) XtPopdown($current_menu);
    $current_menu = menu;
    XtVaGetValues(w, XtNmenuCursor, &cursor, NULL);
    XtGrabPointer($, True, EnterWindowMask | LeaveWindowMask |
		  ButtonPressMask | ButtonReleaseMask, GrabModeAsync,
		  GrabModeAsync, None, cursor, CurrentTime);
    XtAddGrab($, True, False);
}

@ When a menu is popped down again,the |menu_popdown_cb| function
will be called. It removes the MenuBarwidget from the cascade list
and resets |current_menu| to |NULL|.

@proc menu_popdown_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
    Widget $ = (Widget) client_data;

    $current_menu = NULL;
    /* XtRemoveGrab($); */
    XtUngrabPointer($, CurrentTime);
}

@IMPORTS

@ For the definition of |XtNpopup| we need the header file for the
PullDown widget.

@incl <Xfwf/PullDown.h>
@incl <stdio.h>
