# OptButton -- Option button widget
# Bert Bos <bert@let.rug.nl>
# $Ident$

@class XfwfOptionButton (XfwfPullDown)  @file=OptButton

@ The option button is very much like a PullDown button, except that
the label of the button is automatically set to the label of the last
selected menu item.

There are now three ways to specify the menu: with the |popup|
resource, with the |menu| resource, or with the |options| resource.
When the |popup| resource is used, the option button cannot update
its own label, so it becomes the responsibility of the application
that created the pop up widget to set the label. The same goes for the
use of the |menu| resource.

Much more convenient is the |options| resource. It should be set
simply to a newline-separated list of items.

When the user presses the mouse buttonon the button, the
|activateCallback| function is called and then the menu is popped up.
When an item from the menu is chosen, the option button's label will
be set to reflect the choice and the |callback| callbacks will ba
called, with the chosen item as |call_data|.



@PUBLIC

@ One additional callback is defined, simply called |callback|. It is
called when a choice is made from the options menu (and the options
menu has been created with the |options| resource), but only when the
choice is different from the previous choice (as stored in the option
button's label.) The |call_data| argument will contain the new label.

@var <Callback> XtCallbackList callback = NULL


@ The |options| resource can be set to a newline-separated list of
items. If it is set, it will override the |menu| and |popup|
resources.

@var String options = NULL


@ The frame width is again set to a default of 2 pixels.

@var frameWidth = 2



@METHODS

@ The |initialize| method checks if the |options| resource has been
set and if so, creates a matching |menu| resource string.

@proc initialize
{
    if ($options != NULL)
	create_menu_string($);
}


@ The |set_values| method does something similar.

@proc set_values
{
    String s;

    if ($old$options != $options)
	create_menu_string($);
    return False;
}



@TRANSLATIONS

@ Since wbuild cannot yet inherit translations, they must be
given again.

@trans <Btn1Down>: open_menu()
@trans Button1<Enter>: open_menu()
@trans <Key>Return: open_menu()



@UTILITIES

@ The |create_menu_string| routine creates a string in the appropriate
syntax for the |menu| resource. The length is initialized to 1,
because the allocated length must always be one greater than the
length of the string, to accomodate the null byte.

@proc create_menu_string($)
{
    String p, s = NULL;
    int l = 1;

    s = XtMalloc(1);
    s[0]= '\0';
    p = strtok($options, "\n");
    while (p != NULL) {
	l += 2*strlen(p) + strlen("(item \"\" (action \"menu_action()\"))");
	s = XtRealloc(s, l);
	if (s == NULL) XtError("Out of memory");
	(void) strcat(s, "(item \"");
	(void) strcat(s, p);
	(void) strcat(s, "\" (action \"menu_action(");
	(void) strcat(s, p);
	(void) strcat(s, ")\"))");
	p = strtok(NULL, "\n");
    }
    XtVaSetValues($, XtNmenu, s, NULL);
}



@ACTIONS

@ The |menu_action| action routine is called when an option from the
options menu is selected. It will set the label of the option button,
and, if the label is different from the previous one, it will also
call the |callback| functions, with the new label as |call_data|.

@proc menu_action
{
    Widget button = XtParent($);

    if ($button$label && strcmp($button$label, params[0]) == 0) return;
    XtVaSetValues(button, XtNlabel, params[0], NULL);
    XtCallCallbackList(button, $button$callback, $button$label);
}



@IMPORTS

@incl <stdio.h>
